/*
 * Copyright (c) 2017 SpaceToad and the BuildCraft team
 * This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0. If a copy of the MPL was not
 * distributed with this file, You can obtain one at https://mozilla.org/MPL/2.0/
 */

package buildcraft.lib.gui;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import javax.annotation.Nullable;

import com.google.common.collect.ImmutableList;

import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.entity.player.EntityPlayerMP;
import net.minecraft.entity.player.InventoryPlayer;
import net.minecraft.inventory.ClickType;
import net.minecraft.inventory.Container;
import net.minecraft.inventory.Slot;
import net.minecraft.item.ItemStack;
import net.minecraft.util.NonNullList;

import net.minecraftforge.fml.common.network.simpleimpl.MessageContext;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.items.IItemHandler;
import net.minecraftforge.items.IItemHandlerModifiable;

import buildcraft.api.core.BCDebugging;
import buildcraft.api.core.BCLog;

import buildcraft.lib.gui.slot.IPhantomSlot;
import buildcraft.lib.gui.slot.SlotPhantom;
import buildcraft.lib.misc.StackUtil;
import buildcraft.lib.misc.data.IdAllocator;
import buildcraft.lib.net.IPayloadWriter;
import buildcraft.lib.net.MessageContainer;
import buildcraft.lib.net.MessageManager;
import buildcraft.lib.net.PacketBufferBC;
import buildcraft.lib.tile.item.IItemHandlerAdv;

public abstract class ContainerBC_Neptune extends Container {
    public static final boolean DEBUG = BCDebugging.shouldDebugLog("lib.container");

    protected static final IdAllocator IDS = new IdAllocator("container");
    /** Generic "data" id. Used by all containers which only have 1 id to write out (no point in making EVERY container
     * have an {@link IdAllocator} if they only allocate one. */
    public static final int NET_DATA = IDS.allocId("DATA");
    public static final int NET_WIDGET = IDS.allocId("WIDGET");
    public static final int NET_SET_PHANTOM = IDS.allocId("SET_PHANTOM");
    public static final int NET_SET_PHANTOM_MULTI = IDS.allocId("NET_SET_PHANTOM_MULTI");

    public final EntityPlayer player;
    private final List<Widget_Neptune<?>> widgets = new ArrayList<>();

    public ContainerBC_Neptune(EntityPlayer player) {
        this.player = player;
    }

    /** @return The {@link IdAllocator} that allocates all ID's for this class, and its parent classes. All subclasses
     *         should override this if they allocate their own ids after calling
     *         {@link IdAllocator#makeChild(String)} */
    public IdAllocator getIdAllocator() {
        return IDS;
    }

    protected void addFullPlayerInventory(int startX, int startY) {
        for (int sy = 0; sy < 3; sy++) {
            for (int sx = 0; sx < 9; sx++) {
                func_75146_a(new Slot(player.field_71071_by, sx + sy * 9 + 9, startX + sx * 18, startY + sy * 18));
            }
        }

        for (int sx = 0; sx < 9; sx++) {
            func_75146_a(new Slot(player.field_71071_by, sx, startX + sx * 18, startY + 58));
        }
    }

    protected void addFullPlayerInventory(int startY) {
        addFullPlayerInventory(8, startY);
    }

    protected <W extends Widget_Neptune<?>> W addWidget(W widget) {
        if (widget == null) throw new NullPointerException("widget");
        widgets.add(widget);
        return widget;
    }

    public ImmutableList<Widget_Neptune<?>> getWidgets() {
        return ImmutableList.copyOf(widgets);
    }

    @Nullable
    @Override
    public ItemStack func_184996_a(int slotId, int dragType, ClickType clickType, EntityPlayer player) {
        Slot slot = slotId < 0 ? null : this.field_75151_b.get(slotId);
        if (slot == null) {
            return super.func_184996_a(slotId, dragType, clickType, player);
        }

        ItemStack playerStack = player.field_71071_by.func_70445_o();
        if (slot instanceof IPhantomSlot) {
            IPhantomSlot phantom = (IPhantomSlot) slot;
            if (playerStack.func_190926_b()) {
                slot.func_75215_d(ItemStack.field_190927_a);
            } else if (!StackUtil.canMerge(playerStack, StackUtil.asNonNull(slot.func_75211_c()))) {
                ItemStack copy = playerStack.func_77946_l();
                copy.func_190920_e(1);
                slot.func_75215_d(copy);
            } else if (phantom.canAdjustCount()) {
                ItemStack stack = slot.func_75211_c();
                if (stack.func_190916_E() < stack.func_77976_d()) {
                    stack.func_190917_f(1);
                    slot.func_75215_d(stack);
                }
            }
            return playerStack;
        }
        return super.func_184996_a(slotId, dragType, clickType, player);
    }

    @Override
    public ItemStack func_82846_b(EntityPlayer playerIn, int index) {
        ItemStack itemstack = ItemStack.field_190927_a;
        Slot slot = this.field_75151_b.get(index);
        Slot firstSlot = this.field_75151_b.get(0);
        int playerInventorySize = 36;
        boolean playerInventoryFirst = firstSlot.field_75224_c instanceof InventoryPlayer;

        if (slot != null && slot.func_75216_d()) {
            ItemStack itemstack1 = slot.func_75211_c();
            itemstack = itemstack1.func_77946_l();

            if (field_75151_b.size() == playerInventorySize) return ItemStack.field_190927_a;
            if (playerInventoryFirst) {
                if (index < playerInventorySize) {
                    if (!this.func_75135_a(itemstack1, playerInventorySize, this.field_75151_b.size(), false)) {
                        return ItemStack.field_190927_a;
                    }
                } else if (!this.func_75135_a(itemstack1, 0, playerInventorySize, true)) {
                    return ItemStack.field_190927_a;
                }
            } else {
                if (index < this.field_75151_b.size() - playerInventorySize) {
                    if (!this.func_75135_a(itemstack1, this.field_75151_b.size() - playerInventorySize,
                        this.field_75151_b.size(), false)) {
                        return ItemStack.field_190927_a;
                    }
                } else if (!this.func_75135_a(itemstack1, 0, this.field_75151_b.size() - playerInventorySize,
                    true)) {
                    return ItemStack.field_190927_a;
                }
            }

            if (itemstack1.func_190926_b()) {
                slot.func_75215_d(ItemStack.field_190927_a);
            } else {
                slot.func_75218_e();
            }
        }

        return itemstack;
    }

    public static ItemStack safeCopy(ItemStack in) {
        return in == null ? null : in.func_77946_l();
    }

    // Package-private so that the widget itself can send this
    void sendWidgetData(Widget_Neptune<?> widget, IPayloadWriter writer) {
        int widgetId = widgets.indexOf(widget);
        if (widgetId == -1) {
            if (DEBUG) {
                throw new IllegalArgumentException(
                    "Invalid Widget Request! (" + (widget == null ? "null" : widget.getClass()) + ")");
            } else {
                BCLog.logger.warn("[lib.container] Received an invalid widget sending request!");
                BCLog.logger
                    .warn("[lib.container]   Widget {id = " + widgetId + ", class = " + widget.getClass() + "}");
                BCLog.logger.warn("[lib.container]   Container {class = " + getClass() + "}");
                BCLog.logger.warn(
                    "[lib.container]   Player {class = " + player.getClass() + ", name = " + player.func_70005_c_() + "}");
            }
        } else {
            sendMessage(NET_WIDGET, (buffer) -> {
                buffer.writeShort(widgetId);
                writer.write(buffer);
            });
        }
    }

    public final void sendMessage(int id) {
        Side side = player.field_70170_p.field_72995_K ? Side.CLIENT : Side.SERVER;
        sendMessage(id, (buffer) -> writeMessage(id, buffer, side));
    }

    public final void sendMessage(int id, IPayloadWriter writer) {
        PacketBufferBC payload = PacketBufferBC.write(writer);
        MessageContainer message = new MessageContainer(field_75152_c, id, payload);
        if (player.field_70170_p.field_72995_K) {
            MessageManager.sendToServer(message);
        } else {
            MessageManager.sendTo(message, (EntityPlayerMP) player);
        }
    }

    public void writeMessage(int id, PacketBufferBC buffer, Side side) {}

    public void readMessage(int id, PacketBufferBC buffer, Side side, MessageContext ctx) throws IOException {
        if (id == NET_WIDGET) {
            int widgetId = buffer.readUnsignedShort();
            if (widgetId < 0 || widgetId >= widgets.size()) {
                if (DEBUG) {
                    String string = "Received unknown or invalid widget ID " + widgetId + " on side " + side;
                    if (side == Side.SERVER) {
                        string += " (for player " + player.func_70005_c_() + ")";
                    }
                    BCLog.logger.warn(string);
                }
            } else {
                Widget_Neptune<?> widget = widgets.get(widgetId);
                if (side == Side.SERVER) {
                    widget.handleWidgetDataServer(ctx, buffer);
                } else if (side == Side.CLIENT) {
                    widget.handleWidgetDataClient(ctx, buffer);
                }
            }
        } else if (side == Side.SERVER) {
            if (id == NET_SET_PHANTOM) {
                readSingleSetPhantom(buffer, ctx);
            } else if (id == NET_SET_PHANTOM_MULTI) {
                int count = buffer.readUnsignedByte();
                for (int i = 0; i < count; i++) {
                    readSingleSetPhantom(buffer, ctx);
                }
            }
        }
    }

    private void readSingleSetPhantom(PacketBufferBC buffer, MessageContext ctx) throws IOException {
        int idx = buffer.func_150792_a();
        ItemStack stack = buffer.func_150791_c();
        if (idx >= 0 && idx < field_75151_b.size()) {
            Slot s = field_75151_b.get(idx);
            if (s instanceof SlotPhantom) {
                SlotPhantom ph = (SlotPhantom) s;
                IItemHandlerAdv handler = ph.itemHandler;
                if (handler instanceof IItemHandlerModifiable && handler.canSet(ph.handlerIndex, stack)) {
                    ((IItemHandlerModifiable) handler).setStackInSlot(ph.handlerIndex, stack);
                } else {
                    // log rather than throw an exception because of bugged/naughty clients
                    String s2 = "[lib.container] Received an illegal phantom slot setting request! ";
                    s2 += "[The item handler disallowed the replacement] (Client = ";
                    s2 += ctx.getServerHandler().field_147369_b.func_70005_c_() + ", slot_index = " + idx;
                    s2 += ", stack = " + stack + ")";
                    BCLog.logger.warn(s2);
                }
                return;
            }
        }

        // log rather than throw an exception because of bugged/naughty clients
        String s2 = "[lib.container] Received an illegal phantom slot setting request! ";
        s2 += "[Didn't find a phantom slot for the given index] (Client = ";
        s2 += ctx.getServerHandler().field_147369_b.func_70005_c_() + ", slot_index = " + idx;
        s2 += ", stack = " + stack + ")";
        BCLog.logger.warn(s2);
    }

    /** @throws IllegalArgumentException if a {@link SlotPhantom} couldn't be found with that handler and index */
    public void sendSetPhantomSlot(IItemHandler handler, int index, ItemStack to) {
        sendSetPhantomSlot(findPhantomSlot(handler, index), to);
    }

    /** @param stacks The list of stacks to send. NOTE: this list CAN include nulls -- that indicates that the item
     *            should not be changed.
     * @throws IllegalArgumentException if {@link List#size() stacks.size()} differs from {@link IItemHandler#getSlots()
     *             handler.getSlots()}, or if a {@link SlotPhantom} couldn't be found for that handler and any of the
     *             indexes associated with it. */
    public void sendSetPhantomSlots(IItemHandler handler, List<ItemStack> stacks) {
        if (handler.getSlots() < stacks.size()) {
            throw new IllegalStateException("Too many ItemStacks's in the list to change, compared to the "
                + "size of the inventory! (list = " + stacks + ", handler = " + handler + ")");
        }
        int[] indexes = new int[stacks.size()];
        NonNullList<ItemStack> destinationStacks = NonNullList.func_191196_a();
        int i2 = 0;
        for (int i = 0; i < stacks.size(); i++) {
            ItemStack stack = stacks.get(i);
            if (stack == null) {
                continue;
            }
            destinationStacks.add(stack);
            indexes[i2] = findPhantomSlot(handler, i);
            i2++;
        }
        indexes = Arrays.copyOf(indexes, i2);
        sendSetPhantomSlots(indexes, destinationStacks);
    }

    /** @throws IllegalArgumentException if a phantom slot cannot be found */
    private int findPhantomSlot(IItemHandler handler, int index) {
        int i = 0;
        for (Slot slot : field_75151_b) {
            if (slot instanceof SlotPhantom) {
                SlotPhantom ph = (SlotPhantom) slot;
                if (ph.itemHandler == handler && ph.handlerIndex == index) {
                    return i;
                }
            }
            i++;
        }
        throw new IllegalArgumentException("Couldn't find a slot for " + index + " @ " + handler + " in " + getClass());
    }

    public void sendSetPhantomSlot(SlotPhantom slot, ItemStack to) {
        int index = field_75151_b.indexOf(slot);
        if (index == -1) {
            throw new IllegalArgumentException("Couldn't find a slot for " + slot + " in " + getClass());
        }
        sendSetPhantomSlot(index, to);
    }

    private void sendSetPhantomSlot(int phIndex, ItemStack to) {
        sendMessage(NET_SET_PHANTOM, (buffer) -> {
            buffer.func_150787_b(phIndex);
            buffer.func_150788_a(to);
        });
    }

    private void sendSetPhantomSlots(int[] indexes, NonNullList<ItemStack> stacks) {
        if (indexes.length != stacks.size()) {
            throw new IllegalArgumentException("Sizes don't match! (" + indexes.length + " vs " + stacks.size() + ")");
        }
        sendMessage(NET_SET_PHANTOM_MULTI, (buffer) -> {
            buffer.writeByte(indexes.length);
            for (int i = 0; i < indexes.length; i++) {
                int index = indexes[i];
                ItemStack stack = stacks.get(i);
                buffer.func_150787_b(index);
                buffer.func_150788_a(stack);
            }
        });
    }
}
