/*
 * Copyright (c) 2017 SpaceToad and the BuildCraft team
 * This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0. If a copy of the MPL was not
 * distributed with this file, You can obtain one at https://mozilla.org/MPL/2.0/
 */

package buildcraft.lib.misc;

import java.util.HashMap;
import java.util.Map;

import com.mojang.authlib.GameProfile;
import com.mojang.authlib.minecraft.MinecraftProfileTexture;
import com.mojang.authlib.minecraft.MinecraftProfileTexture.Type;

import net.minecraft.client.Minecraft;
import net.minecraft.client.renderer.texture.TextureAtlasSprite;
import net.minecraft.client.renderer.texture.TextureMap;
import net.minecraft.tileentity.TileEntitySkull;
import net.minecraft.util.ResourceLocation;

import buildcraft.api.core.render.ISprite;

import buildcraft.lib.BCLibSprites;
import buildcraft.lib.client.sprite.SpriteRaw;

public class SpriteUtil {

    private static final Map<GameProfile, GameProfile> CACHED = new HashMap<>();

    public static void bindBlockTextureMap() {
        bindTexture(TextureMap.field_110575_b);
    }

    public static void bindTexture(String identifier) {
        bindTexture(new ResourceLocation(identifier));
    }

    public static void bindTexture(ResourceLocation identifier) {
        Minecraft.func_71410_x().field_71446_o.func_110577_a(identifier);
    }

    /** Transforms the given {@link ResourceLocation}, adding ".png" to the end and prepending that
     * {@link ResourceLocation#getResourcePath()} with "textures/", just like what {@link TextureMap} does. */
    public static ResourceLocation transformLocation(ResourceLocation location) {
        return new ResourceLocation(location.func_110624_b(), "textures/" + location.func_110623_a() + ".png");
    }

    public static ISprite getFaceSprite(GameProfile profile) {
        if (profile == null) {
            return BCLibSprites.HELP;
        }
        Minecraft mc = Minecraft.func_71410_x();

        if (CACHED.containsKey(profile) && CACHED.get(profile) == null && Math.random() >= 0.99) {
            CACHED.remove(profile);
        }

        if (!CACHED.containsKey(profile)) {
            CACHED.put(profile, TileEntitySkull.func_174884_b(profile));
        }
        GameProfile p2 = CACHED.get(profile);
        if (p2 == null) {
            return BCLibSprites.LOCK;
        }
        profile = p2;
        Map<Type, MinecraftProfileTexture> map = mc.func_152342_ad().func_152788_a(profile);
        MinecraftProfileTexture tex = map.get(Type.SKIN);
        if (tex != null) {
            ResourceLocation loc = mc.func_152342_ad().func_152792_a(tex, Type.SKIN);
            return new SpriteRaw(loc, 8, 8, 8, 8, 64);
        }
        return BCLibSprites.LOADING;
    }

    public static TextureAtlasSprite missingSprite() {
        return Minecraft.func_71410_x().func_147117_R().func_174944_f();
    }
}
